import qs from 'qs'
class FindRouteHelper {
  private _url: string
  /**
   * @return {boolean}
   */
  constructor (url, routes) {
    url = url.trim()
      .replace(/^(\/)/, '')

    this.url = url

    const parts = url.split('/')
    const count = parts.length
    let matches = []
    if (routes.hasOwnProperty(count)) {
      routes[count].forEach((route) => {
        let matched = url.match(route.pattern)
        if (matched) matches.push(route)
      })
    } else {
      return
    }
  }

  get url() {
    return this._url || ''
  }

  set url(value) {
    this._url = value
  }

  static sortByWeight (suggested) {
    suggested.sort((a, b) => {
      return (b.weight - a.weight)
    })
    return suggested[0]
  }

  parseQuery () {
    return qs.parse(this.url.replace(/^.*\?/, ''))
  }

  parseParams (parts, segments) {
    let params = {}
    segments.forEach((segment, index) => {
      if (segment.charAt(0) === ':') {
        params[segment.substr(1)] = parts[index]
      }
    })
    return params
  }

  findRoute (parts, routes) {
    switch (routes.length) {
      case 0:
        return false
      case 1: {
        return {
          route: routes[0].route,
          query: this.parseQuery(),
          params: this.parseParams(parts, routes[0].segments),
          meta: routes[0].meta
        }
      }
      default: {
        routes.forEach((routeInfo) => {
          routeInfo.weight = this.calcSuggestedWeight(parts, routeInfo.segments)
        })
        const route = FindRouteHelper.sortByWeight(routes)
        return {
          route: route.route,
          query: this.parseQuery(),
          params: this.parseParams(parts, route.segments),
          meta: route.meta
        }
      }
    }
  }

  calcSuggestedWeight (parts, route) {
    let weight = 0
    parts.forEach((part, index) => {
      if (part === route[index]) {
        weight += 400
      } else {
        if (route[index].charAt(0) !== ':') {
          weight -= 400
        } else {
          weight += 100
        }
      }
    })
    return weight
  }
}

export default (url, routes) => {
  return new FindRouteHelper(url, routes)
}
